/** Define Kerrighed Capabilities (not exported outside kernel)
 * @author Jean Parpaillon (c) Inria 2006
 */

#ifndef _KERRIGHED_CAPABILITIES_H_INTERNAL
#define _KERRIGHED_CAPABILITIES_H_INTERNAL

#include <linux/capability.h>
#include "sys/capabilities.h"

/*
 * MACROS
 */
#define KRG_CAP_INIT_PERM_SET CAP_FULL_SET
#define KRG_CAP_INIT_EFF_SET CAP_TO_MASK (CAP_CHANGE_KERRIGHED_CAP)
#define KRG_CAP_INIT_INH_PER_SET KRG_CAP_INIT_PERM_SET
#define KRG_CAP_INIT_INH_EFF_SET KRG_CAP_INIT_EFF_SET

#define KRG_CAP_INIT_ATOMIC_COUNTERS \
	{{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0},{0}}

#define can_use_krg_cap(task,cap) \
	(cap_raised(task->krg_cap_effective,cap) \
	 && !atomic_read(&task->krg_cap_unavailable[cap]) \
	 && !atomic_read(&task->krg_cap_unavailable_private[cap]))

#define DEFAULT_KRG_DAEMON_CAP_EFF 0
#define DEFAULT_KRG_DAEMON_CAP_PERM 0

/*
 * TYPES
 */
#ifdef __KERNEL__

struct krg_cap_struct
{
	kernel_cap_t krg_cap_effective;
	kernel_cap_t krg_cap_permitted;
	kernel_cap_t krg_cap_inheritable_permitted;
	kernel_cap_t krg_cap_inheritable_effective;
};

#else

#include <sys/types.h>

#endif /* __KERNEL__ */

struct krg_cap_pid_desc
{
	pid_t pid;
	krg_cap_t *caps;
};

#endif /* _KERRIGHED_CAPABILITIES_H_INTERNAL */
