#ifndef __KKRG_SHM__
#define __KKRG_SHM__

#include <linux/msg.h>
#include <../ipc/util.h>

struct vm_area_struct;


/** Kerrighed Hooks **/

extern struct shmid_kernel * (*kh_shm_lock)(struct ipc_namespace *ns, int id);
extern void (*kh_shm_unlock)(struct shmid_kernel *shp);

extern int (*kh_shm_newseg) (struct ipc_namespace *ns, 
			     struct shmid_kernel * shp);
extern void (*kh_shm_destroy)(struct ipc_namespace *ns,
			      struct shmid_kernel *shp) ;
extern void (*kh_shm_rmkey)(struct ipc_namespace *ns,
			   struct shmid_kernel *shp) ;
extern int (*kh_shm_get_max_id)(struct ipc_namespace *ns, struct ipc_ids* ids);
extern int (*kh_get_ipc_id)(struct ipc_ids* ids, int size);
extern int (*kh_ipc_findkey)(struct ipc_ids* ids, key_t key);


/** Exported variables  **/

extern struct vm_operations_struct krg_shmem_vmops;
extern struct file_operations shm_file_operations ;



/** Exported functions  **/

#define shm_ids(ns)	(*((ns)->ids[IPC_SHM_IDS]))
#define shm_file_ns(file) (*((struct ipc_namespace **)&(file)->private_data))

int shm_addid(struct ipc_namespace *ns, struct shmid_kernel *shp);
void shm_inc(struct ipc_namespace *ns, int id);
void shm_close (struct vm_area_struct *shmd);

static inline struct shmid_kernel* local_shm_lock (struct ipc_namespace *ns,
						   int id)
{
       return (struct shmid_kernel*)ipc_lock(&shm_ids(ns),id);
}

static inline struct shmid_kernel* shm_lock (struct ipc_namespace *ns, int id)
{
       if (kh_shm_lock)
               return kh_shm_lock(ns, id);
       else
               return local_shm_lock(ns, id);
}

static inline void local_shm_unlock(struct shmid_kernel *shp)
{
       ipc_unlock(&(shp)->shm_perm);
}
static inline void shm_unlock(struct shmid_kernel *shp)
{
       if (kh_shm_unlock)
               kh_shm_unlock(shp);
       local_shm_unlock(shp);
}

void _shm_destroy(struct ipc_namespace *ns, struct shmid_kernel *shp);

static inline void shm_destroy(struct ipc_namespace *ns,
			       struct shmid_kernel *shp)
{
	if (kh_shm_destroy) {
		shp->locked_for_del = 1;
		kh_shm_destroy(ns, shp);
	}
	else
		_shm_destroy(ns, shp);
}

#define shm_get(ns, id)			\
	((struct shmid_kernel*)ipc_get(&shm_ids(ns),id))
#define shm_buildid(ns, id, seq)	\
	ipc_buildid(&shm_ids(ns), id, seq)

#endif // __KKRG_SHM__
