/*
 *  Kerrighed/kernel/kerrighed/epm.c
 *
 *  Copyright (C) 2007 Louis Rilling - Kerlabs
 */

#include <linux/module.h>

/* Kerrighed hooks */

#include <linux/types.h>
#include <kerrighed/sys/types.h>
#include <kerrighed/krgsyms.h>
#include <kerrighed/kerrighed_signal.h>
#include <asm/signal.h>

/* remote clone */
struct pt_regs;

int (*kh_do_fork)(unsigned long clone_flags,
		  unsigned long stack_start,
		  struct pt_regs *regs,
		  unsigned long stack_size,
		  int *parent_tidptr,
		  int *child_tidptr) = NULL;
EXPORT_SYMBOL(kh_do_fork);

/* remote syscalls */
#if 0 /* Temporarily disabled */
int (*kh_setpgid)(pid_t pgid, pid_t session);
#endif /* 0 */
int (*kh_forward_setpgid)(pid_t pid, pid_t pgid);
#if 0 /* Temporarily disabled */
EXPORT_SYMBOL(kh_setpgid);
#endif /* 0 */
EXPORT_SYMBOL(kh_forward_setpgid);

/* parent for processes whose real parent is on a distant node */
struct task_struct;

struct task_struct *baby_sitter = NULL;
EXPORT_SYMBOL(baby_sitter);

/* Kerrighed signal */
kerrighed_handler_t *kh_krg_handler[_NSIG];
EXPORT_SYMBOL(kh_krg_handler);

/* pid reference tracking */
struct pid;
struct pid * (*kh_get_pid)(int nr);
void (*kh_end_get_pid)(struct pid *pid);
void (*kh_put_pid)(struct pid *pid);
/* EXPORT_SYMBOL(__free_pidmap); */
EXPORT_SYMBOL(kh_get_pid);
EXPORT_SYMBOL(kh_end_get_pid);
EXPORT_SYMBOL(kh_put_pid);

/* pgrp */
int (*kh_will_become_orphaned_pgrp)(pid_t pgrp, pid_t ignored_kid) = NULL;
EXPORT_SYMBOL(kh_will_become_orphaned_pgrp);

/* children */
struct children_ctnr_object;

struct children_ctnr_object * (*kh_alloc_children)(struct task_struct *task);
void (*kh_share_children)(struct task_struct *task);
void (*kh_exit_children)(struct task_struct *task);
void (*kh_children_get)(struct children_ctnr_object *obj);
void (*kh_children_put)(struct children_ctnr_object *obj);
int (*kh_children_alive)(struct children_ctnr_object *obj);
int (*kh_new_child)(struct children_ctnr_object *obj, pid_t parent_pid,
		    pid_t child_pid, pid_t pgid, pid_t sid,
		    int exit_signal);
void (*kh_set_child_pgid)(struct children_ctnr_object *obj,
			  pid_t pid, pid_t pgid);
void (*kh_set_child_exit_signal)(struct children_ctnr_object *obj,
				 pid_t pid, int exit_signal);
void (*kh_remove_child)(struct children_ctnr_object *obj, pid_t child_pid);
void (*kh_reparent_child)(struct children_ctnr_object *obj, pid_t child_pid,
			  pid_t reaper_pid, int same_group);
void (*kh_forget_original_remote_parent)(struct task_struct *parent,
					 struct task_struct *reaper);
void (*kh_reparent_to_local_child_reaper)(struct task_struct *task);
int (*kh_may_be_eligible_child)(struct children_ctnr_object *obj,
				pid_t pid,
				int options);
pid_t (*kh_get_real_parent_tgid)(struct task_struct *task);
int (*kh_get_parent)(struct children_ctnr_object *obj, pid_t pid,
		     pid_t *parent_pid, pid_t *real_parent_pid);
void (*kh_ptrace_link)(struct children_ctnr_object *obj,
		       pid_t ptracer_pid,
		       pid_t child_pid);
void (*kh_ptrace_unlink)(struct children_ctnr_object *obj, pid_t child_pid);
struct children_ctnr_object * (*kh_children_writelock)(pid_t tgid);
struct children_ctnr_object * (*kh_children_writelock_nested)(pid_t tgid);
struct children_ctnr_object * (*kh_children_readlock)(pid_t tgid);
struct children_ctnr_object * (*kh_parent_children_writelock)(
	struct task_struct *task,
	pid_t *parent_tgid);
struct children_ctnr_object * (*kh_parent_children_readlock)(
	struct task_struct *task,
	pid_t *parent_tgid);
void (*kh_children_unlock)(pid_t tgid);
void (*kh_add_parent)(struct task_struct *task, struct task_struct *parent);
void (*kh_remove_parent)(struct task_struct *task);
EXPORT_SYMBOL(kh_alloc_children);
EXPORT_SYMBOL(kh_share_children);
EXPORT_SYMBOL(kh_exit_children);
EXPORT_SYMBOL(kh_children_get);
EXPORT_SYMBOL(kh_children_put);
EXPORT_SYMBOL(kh_children_alive);
EXPORT_SYMBOL(kh_new_child);
EXPORT_SYMBOL(kh_set_child_pgid);
EXPORT_SYMBOL(kh_set_child_exit_signal);
EXPORT_SYMBOL(kh_remove_child);
EXPORT_SYMBOL(kh_reparent_child);
EXPORT_SYMBOL(kh_forget_original_remote_parent);
EXPORT_SYMBOL(kh_reparent_to_local_child_reaper);
EXPORT_SYMBOL(kh_may_be_eligible_child);
EXPORT_SYMBOL(kh_get_real_parent_tgid);
EXPORT_SYMBOL(kh_get_parent);
EXPORT_SYMBOL(kh_ptrace_link);
EXPORT_SYMBOL(kh_ptrace_unlink);
EXPORT_SYMBOL(kh_children_writelock);
EXPORT_SYMBOL(kh_children_writelock_nested);
EXPORT_SYMBOL(kh_children_readlock);
EXPORT_SYMBOL(kh_parent_children_writelock);
EXPORT_SYMBOL(kh_parent_children_readlock);
EXPORT_SYMBOL(kh_children_unlock);
EXPORT_SYMBOL(kh_add_parent);
EXPORT_SYMBOL(kh_remove_parent);

/* ptrace */
int (*kh_ptrace_traceme)(void);
int (*kh_ptrace_attach)(struct task_struct *task) = NULL;
void (*kh_ptrace_detach)(struct task_struct *task) = NULL;
void (*kh_ptrace_ptracer_exit)(struct task_struct *task);
void (*kh_ptrace_reparent_ptraced)(struct task_struct *real_parent,
				   struct task_struct *task);
void (*kh_ptrace_release_ptraced)(struct task_struct *task);
void (*kh_ptrace_copy_ptrace)(struct task_struct *task);
EXPORT_SYMBOL(kh_ptrace_traceme);
EXPORT_SYMBOL(kh_ptrace_attach);
EXPORT_SYMBOL(kh_ptrace_detach);
EXPORT_SYMBOL(kh_ptrace_ptracer_exit);
EXPORT_SYMBOL(kh_ptrace_reparent_ptraced);
EXPORT_SYMBOL(kh_ptrace_release_ptraced);
EXPORT_SYMBOL(kh_ptrace_copy_ptrace);

/* exit */
int (*kh_exit_notify)(struct task_struct *task,
		      kerrighed_node_t parent_node,
		      int sig_num,
		      pid_t groupid, pid_t sessionid) = NULL;
void (*kh_delay_release_task)(struct task_struct *task);
EXPORT_SYMBOL(kh_exit_notify);
EXPORT_SYMBOL(kh_delay_release_task);

/* signal handlers sharing */
struct sighand_struct;

struct sighand_struct* (*kh_malloc_sighand_struct)(struct task_struct *task,
						   int need_update) = NULL;
void (*kh_share_sighand)(struct task_struct *task);
unsigned long (*kh_exit_sighand)(unsigned long id);
struct sighand_struct * (*kh_sighand_struct_readlock)(unsigned long id) = NULL;
struct sighand_struct * (*kh_sighand_struct_writelock)(unsigned long id) = NULL;
void (*kh_sighand_struct_unlock)(unsigned long id) = NULL;
EXPORT_SYMBOL(kh_malloc_sighand_struct);
EXPORT_SYMBOL(kh_share_sighand);
EXPORT_SYMBOL(kh_exit_sighand);
EXPORT_SYMBOL(kh_sighand_struct_readlock);
EXPORT_SYMBOL(kh_sighand_struct_writelock);
EXPORT_SYMBOL(kh_sighand_struct_unlock);

/* migration */
int (*kh_unset_pid_location)(pid_t pid) = NULL;
EXPORT_SYMBOL(kh_unset_pid_location);

/* Krgsyms to register for restart_blocks in ghost processes */
extern int compat_krgsyms_register(void);
extern int posix_timers_krgsyms_register(void);
extern int hrtimer_krgsyms_register(void);
extern int posix_cpu_timers_krgsyms_register(void);
extern int compat_krgsyms_unregister(void);
extern int posix_timers_krgsyms_unregister(void);
extern int hrtimer_krgsyms_unregister(void);
extern int posix_cpu_timers_krgsyms_unregister(void);

int restart_block_krgsyms_register(void)
{
	int retval;

	retval = krgsyms_register(KRGSYMS_DO_NO_RESTART_SYSCALL,
				  do_no_restart_syscall);
#ifdef CONFIG_COMPAT
	if (!retval)
		retval = compat_krgsyms_register();
#endif
	if (!retval)
		retval = posix_timers_krgsyms_register();
	if (!retval)
		retval = hrtimer_krgsyms_register();
	if (!retval)
		retval = posix_cpu_timers_krgsyms_register();

	return retval;
}

int restart_block_krgsyms_unregister(void)
{
	int retval;

	retval = krgsyms_unregister(KRGSYMS_DO_NO_RESTART_SYSCALL);
#ifdef CONFIG_COMPAT
	if (!retval)
		retval = compat_krgsyms_unregister();
#endif
	if (!retval)
		retval = posix_timers_krgsyms_unregister();
	if (!retval)
		retval = hrtimer_krgsyms_unregister();
	if (!retval)
		retval = posix_cpu_timers_krgsyms_unregister();

	return retval;
}

EXPORT_SYMBOL(restart_block_krgsyms_register);
EXPORT_SYMBOL(restart_block_krgsyms_unregister);

/* vanilla Linux symbols */

#include <linux/syscalls.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/thread_info.h>
#include <linux/syscalls.h>
#include <linux/nsproxy.h>
#include <linux/mount.h>
#include <linux/mnt_namespace.h>
#include <linux/pid_namespace.h>
#include <kerrighed/syscalls.h>
#include <kerrighed/signal.h>

extern void *_stext;
extern void *_etext;

/* remote syscalls */
EXPORT_SYMBOL(sys_setpgid);

/* ghost process */
EXPORT_SYMBOL(wake_up_new_task);
EXPORT_SYMBOL(alloc_uid);
EXPORT_SYMBOL(free_uid);
EXPORT_SYMBOL(default_exec_domain);
EXPORT_SYMBOL(do_no_restart_syscall);
EXPORT_SYMBOL(vfsmount_lock);
EXPORT_SYMBOL(__put_mnt_ns);
EXPORT_SYMBOL(init_pid_ns);
EXPORT_SYMBOL(free_pid_ns);
EXPORT_SYMBOL(free_nsproxy);

/* PID allocation */
EXPORT_SYMBOL(alloc_pid);
EXPORT_SYMBOL(free_pid);

/* exit */
EXPORT_SYMBOL(do_notify_parent);
EXPORT_SYMBOL(release_task);

EXPORT_SYMBOL(sighand_cachep);

/* used by checkpoint */
EXPORT_SYMBOL(sys_unlink);
EXPORT_SYMBOL(sys_mkdir);

/* only used by DEBUGs */
EXPORT_SYMBOL(_stext);
EXPORT_SYMBOL(_etext);
