/*
 *  Copyright (C) 2006-2007, Renaud Lottiaux, Pascal Gallard, Kerlabs.
 */

#include <linux/module.h>
#include <linux/socket.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/syscalls.h>


/** Pure file system Kerrighed Kernel Hooks **/

off_t (*kh_faf_lseek)(struct file *file, off_t offset,
			     unsigned int origin) ;
long (*kh_faf_llseek)(struct file *file, unsigned long offset_high,
			     unsigned long offset_low, loff_t * result,
			     unsigned int origin) ;
ssize_t (*kh_faf_read)(struct file *file, char * buf, size_t count) ;
ssize_t (*kh_faf_write)(struct file *file, const char * buf,
			size_t count) ;
long (*kh_faf_fcntl)(struct file *file, unsigned int cmd,
		     unsigned long arg) ;
long (*kh_faf_fcntl64)(struct file *file, unsigned int cmd,
		       unsigned long arg) ;
long (*kh_faf_ioctl)(struct file *file, unsigned int cmd,
		     unsigned long arg) ;
long (*kh_faf_fstat)(struct file *file, struct kstat *stat) ;
long (*kh_faf_fsync)(struct file *file) ;
long (*kh_faf_flock)(struct file *file, unsigned int cmd);
void (*kh_faf_get_file)(struct file *file);
char *(*kh_faf_d_path)(struct file *file, char *buffer, int size);
int (*kh_faf_close_srv_file)(struct file *file);



/** Communication related Kerrighed Kernel Hooks **/

long (*kh_faf_bind)(struct file * file, struct sockaddr __user *umyaddr,
		    int addrlen);
long (*kh_faf_connect)(struct file * file, struct sockaddr __user *uservaddr,
		       int addrlen);
long (*kh_faf_listen)(struct file * file, int backlog);
long (*kh_faf_accept)(struct file * file,
		      struct sockaddr __user *upeer_sockaddr,
		      int __user *upeer_addrlen);
long (*kh_faf_getsockname)(struct file * file,
			   struct sockaddr __user *usockaddr,
			   int __user *usockaddr_len);
long (*kh_faf_getpeername)(struct file * file, 
			   struct sockaddr __user *usockaddr,
			   int __user *usockaddr_len);
long (*kh_faf_send)(struct file * file, void __user * buff, size_t len,
		    unsigned flags);
long (*kh_faf_sendto)(struct file * file, void __user * buff, size_t len,
		      unsigned flags, struct sockaddr __user *addr, 
		      int addr_len);
long (*kh_faf_recv)(struct file * file, void __user * ubuf, size_t size,
		    unsigned flags);
long (*kh_faf_recvfrom)(struct file * file, void __user * ubuf, size_t size,
			unsigned flags, struct sockaddr __user *addr,
			int __user *addr_len);
long (*kh_faf_shutdown)(struct file * file, int how);
long (*kh_faf_setsockopt)(struct file * file, int level, int optname,
			  char __user *optval, int optlen);
long (*kh_faf_getsockopt)(struct file * file, int level, int optname,
			  char __user *optval, int __user *optlen);
long (*kh_faf_sendmsg)(struct file * file, struct msghdr __user *msg,
		       unsigned flags);
long (*kh_faf_recvmsg)(struct file * file, struct msghdr __user *msg,
		       unsigned int flags);



/** Kerrighed Exported Symbols **/

EXPORT_SYMBOL(kh_faf_lseek);
EXPORT_SYMBOL(kh_faf_llseek);
EXPORT_SYMBOL(kh_faf_read);
EXPORT_SYMBOL(kh_faf_write);
EXPORT_SYMBOL(kh_faf_fcntl);
EXPORT_SYMBOL(kh_faf_fcntl64);
EXPORT_SYMBOL(kh_faf_ioctl);
EXPORT_SYMBOL(kh_faf_fstat);
EXPORT_SYMBOL(kh_faf_fsync);
EXPORT_SYMBOL(kh_faf_flock);
EXPORT_SYMBOL(kh_faf_get_file);
EXPORT_SYMBOL(kh_faf_bind);
EXPORT_SYMBOL(kh_faf_connect);
EXPORT_SYMBOL(kh_faf_listen);
EXPORT_SYMBOL(kh_faf_accept);
EXPORT_SYMBOL(kh_faf_getsockname);
EXPORT_SYMBOL(kh_faf_getpeername);
EXPORT_SYMBOL(kh_faf_send);
EXPORT_SYMBOL(kh_faf_sendto);
EXPORT_SYMBOL(kh_faf_recv);
EXPORT_SYMBOL(kh_faf_recvfrom);
EXPORT_SYMBOL(kh_faf_shutdown);
EXPORT_SYMBOL(kh_faf_setsockopt);
EXPORT_SYMBOL(kh_faf_getsockopt);
EXPORT_SYMBOL(kh_faf_sendmsg);
EXPORT_SYMBOL(kh_faf_recvmsg);
EXPORT_SYMBOL(kh_faf_d_path);


EXPORT_SYMBOL(sys_getpeername);
EXPORT_SYMBOL(sys_connect);
EXPORT_SYMBOL(sys_fcntl);
#if BITS_PER_LONG == 32
EXPORT_SYMBOL(sys_fcntl64);
#endif
EXPORT_SYMBOL(sys_sendmsg);
EXPORT_SYMBOL(sys_shutdown);
EXPORT_SYMBOL(sys_llseek);
EXPORT_SYMBOL(sys_ioctl);
EXPORT_SYMBOL(sys_getsockopt);
EXPORT_SYMBOL(sys_getsockname);
EXPORT_SYMBOL(sys_sendto);
EXPORT_SYMBOL(sys_write);
EXPORT_SYMBOL(sys_accept);
EXPORT_SYMBOL(__fd_install);
EXPORT_SYMBOL(sys_send);
EXPORT_SYMBOL(__put_unused_fd);
EXPORT_SYMBOL(sys_fsync);
EXPORT_SYMBOL(sys_flock);
EXPORT_SYMBOL(sys_bind);
EXPORT_SYMBOL(sys_recvmsg);
EXPORT_SYMBOL(kh_faf_close_srv_file);
EXPORT_SYMBOL(sys_listen);
EXPORT_SYMBOL(sys_lseek);
EXPORT_SYMBOL(__get_unused_fd);
EXPORT_SYMBOL(sys_recvfrom);
EXPORT_SYMBOL(sys_setsockopt);
EXPORT_SYMBOL(sys_recv);
EXPORT_SYMBOL(__filp_close);
