/*
 *  Copyright (C) 2006-2007, Kerlabs.
 */

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel_stat.h>
#include <linux/swap.h>
#include <linux/proc_fs.h>
#include <linux/file.h>
#include <linux/mm.h>
#include <linux/syscalls.h>
#include <linux/rmap.h>
#include <linux/acct.h>
#include <linux/pagemap.h>
#include <linux/fs.h>
#include <linux/rwsem.h>
#include <asm/pgalloc.h>
#include <asm/pgtable.h>
#include <asm/tlbflush.h> // Needed for UserMode
#include <asm/unistd.h>
#ifndef CONFIG_USERMODE
#include <asm/sigcontext.h>
#else
#include <sigcontext.h>
#endif

#include <linux/msg.h>

#include <kerrighed/kernel_headers.h>
#include <kerrighed/unique_id.h>
#ifdef CONFIG_KRG_STREAM
#include <kerrighed/net.h>
#endif
#include <kerrighed/hashtable.h>
#include <kerrighed/krgsyms.h>



/*****************************************************************************/
/*                                                                           */
/*                          KERRIGHED KERNEL HOOKS                           */
/*                                                                           */
/*****************************************************************************/


#ifdef CONFIG_KRG_CTNR
int (*kh_copy_kddm_info)(unsigned long clone_flags, struct task_struct * tsk);
EXPORT_SYMBOL(kh_copy_kddm_info);
#endif


#ifdef CONFIG_KRG_SYNC
int (*kh_futex_queue_me)(struct futex_q *q) = NULL;
int (*kh_futex_wake)(union futex_key *key, int nr_wake, int nr_woken) = NULL;
#endif // CONFIG_KRG_SYNC

#ifdef CONFIG_KRG_STREAM
struct proto_ops *kernet_socket_ops;
struct proto *kernet_socket_proto;
struct proto_ops *kernet_unix_stream_ops;
struct proto_ops *kernet_unix_datagram_ops;
struct proto_ops *kernet_inet_stream_ops;
struct proto_ops *kernet_inet_datagram_ops;
#endif

#ifdef CONFIG_KRG_STREAM
#warning deprecated code
#if defined(CONFIG_SMP) || defined(CONFIG_PREEMPT)
extern void net_family_write_lock(void);
extern void net_family_write_unlock(void);
#else
extern struct net_proto_family *net_families[];
#define net_family_write_lock() do { } while(0)
#define net_family_write_unlock() do { } while(0)
#endif
#endif

#ifdef CONFIG_KRG_STREAM_PIPE
void (*setup_krg_read_pipe)(struct file *) = NULL ;
void (*setup_krg_write_pipe)(struct file *) = NULL ;
struct inode * (*get_krg_pipe_inode)(struct vfsmount *) = NULL ;
#endif

/* Kerrighed FS management */

loff_t (*kh_file_pos_read)(struct file *file) = NULL;
void (*kh_file_pos_write)(struct file *file, loff_t pos) = NULL;
void (*kh_put_file)(struct file *file) = NULL;

#ifdef CONFIG_KRG_STREAM_PIPE
int local_stream_flush(struct file * filp)
{
	int i ;
	int file_still_present = 0 ;
	if (current->files && !(current->flags & PF_EXITING))
	{
		for (i = 0 ; i < current->files->max_fds ; i++)
		{
			if ( FD_ISSET(i, current->files->open_fds) )
			{
				if (current->files->fd[i] == filp)
				{
					file_still_present = 1 ;
					break ;
				}
			}
		}
		if (!file_still_present)
		{
			atomic_dec(&current->krg_cap_unavailable[CAP_CAN_MIGRATE]) ;
			if (atomic_read (&current->krg_cap_unavailable[CAP_CAN_MIGRATE]) < 0 )
				BUG() ;
			atomic_dec(&current->krg_cap_unavailable[CAP_DISTANT_FORK]) ;
			if (atomic_read (&current->krg_cap_unavailable[CAP_DISTANT_FORK]) < 0 )
				BUG() ;
		}
	}
	else
	{
		//we are exiting and/or files is NULL : we can't keep count correctly
		atomic_dec(&current->krg_cap_unavailable[CAP_CAN_MIGRATE]) ;
		if (atomic_read (&current->krg_cap_unavailable[CAP_CAN_MIGRATE]) < 0 )
			atomic_set(&current->krg_cap_unavailable[CAP_CAN_MIGRATE], 0) ;
		atomic_dec(&current->krg_cap_unavailable[CAP_DISTANT_FORK]) ;
		if (atomic_read (&current->krg_cap_unavailable[CAP_DISTANT_FORK]) < 0 )
			atomic_set(&current->krg_cap_unavailable[CAP_DISTANT_FORK], 0) ;
	}
	return 0 ;
}
#endif
#if defined(CONFIG_KRG_STREAM_INET) || defined(CONFIG_KRG_STREAM_UNIX)
int sock_change(struct net_proto_family *newops,
		struct net_proto_family **oldops){
  if (newops->family >= NPROTO) {
    printk(KERN_CRIT "protocol %d >= NPROTO(%d)\n", newops->family, NPROTO);
    return -ENOBUFS;
  }

  net_family_write_lock();
  if(oldops) *oldops = net_families[newops->family];
  net_families[newops->family] = newops;
  net_family_write_unlock();

  return 0;
};
#endif


#ifdef CONFIG_FD_V1
/* FORK_DELAY */
int jeton = -1;
int var_chan_admin = -1;
EXPORT_SYMBOL(var_chan_admin);
EXPORT_SYMBOL(jeton);
#endif //CONFIG_FD_V1

#ifdef CONFIG_KRG_FD
int wait_time = 3000;
EXPORT_SYMBOL(wait_time);
int vload = 0;
EXPORT_SYMBOL(vload);
int nb_process_delayed = 0;
EXPORT_SYMBOL(nb_process_delayed);
struct l_p_delay head_list_fd;
EXPORT_SYMBOL(head_list_fd);
int init_fd = 1; //initialization fd
EXPORT_SYMBOL(init_fd);
int (*kh_fd_send_msg)(int code, int type) = NULL;
EXPORT_SYMBOL(kh_fd_send_msg);
int threshold = 1990;
EXPORT_SYMBOL(threshold);
int threshold_scheduler = 2300; //krg_scheduler.c
EXPORT_SYMBOL(threshold_scheduler);
#endif //CONFIG_KRG_FD
/*
 * Kerrighed symbol
 */

#ifdef CONFIG_KRG_SYNC
EXPORT_SYMBOL(kh_futex_queue_me);
EXPORT_SYMBOL(kh_futex_wake);
EXPORT_SYMBOL(__futex_wake);
#endif // CONFIG_KRG_SYNC

#ifdef CONFIG_KERRIGHED
EXPORT_SYMBOL(hashtable_new);
EXPORT_SYMBOL(hashtable_free);
EXPORT_SYMBOL(hashtable_add);
EXPORT_SYMBOL(__hashtable_add);
EXPORT_SYMBOL(hashtable_remove);
EXPORT_SYMBOL(__hashtable_remove);
EXPORT_SYMBOL(hashtable_find);
EXPORT_SYMBOL(__hashtable_find);
EXPORT_SYMBOL(__hashtable_foreach_key);
EXPORT_SYMBOL(__hashtable_foreach_data);
EXPORT_SYMBOL(hashtable_find_data);
EXPORT_SYMBOL(krgsyms_register);
EXPORT_SYMBOL(krgsyms_unregister);
EXPORT_SYMBOL(krgsyms_export);
EXPORT_SYMBOL(krgsyms_import);
#endif // CONFIG_KERRIGHED

#ifdef CONFIG_KRG_STREAM
EXPORT_SYMBOL(kernet_socket_ops);
EXPORT_SYMBOL(kernet_socket_proto);
#endif

#ifdef CONFIG_KRG_STREAM_PIPE
EXPORT_SYMBOL(setup_krg_read_pipe);
EXPORT_SYMBOL(setup_krg_write_pipe);
EXPORT_SYMBOL(get_krg_pipe_inode);
#endif

#ifdef CONFIG_KRG_STREAM_UNIX
EXPORT_SYMBOL(unix_stream_ops);
EXPORT_SYMBOL(kernet_unix_stream_ops);
EXPORT_SYMBOL(kernet_unix_datagram_ops);
#endif

#ifdef CONFIG_KRG_STREAM_INET
EXPORT_SYMBOL(kernet_inet_stream_ops);
EXPORT_SYMBOL(kernet_inet_datagram_ops);
#endif
