/*
 *  Kerrighed/kernel/kerrighed/proc.c
 *
 *  Copyright (C) 2007 Louis Rilling - Kerlabs
 */

#include <linux/module.h>

/* Kerrighed hooks */

#include <linux/types.h>
#include <kerrighed/sys/types.h>

/* remote syscalls */
struct siginfo;
struct sched_param;

/* not implemented yet */
int (*kh_setpriority)(int which, int who, int niceval, int *error) = NULL;
/* not implemented yet */
int (*kh_getpriority)(int which, int who, long *error) = NULL;
/* not implemented yet */
void (*kh_kill_all)(int sig, struct siginfo *info, int *count, int *retval,
		    int tgid) = NULL;
int (*kh_kill_proc_info)(int sig, struct siginfo *info, pid_t pid) = NULL;
int (*kh_kill_pg_info)(int sig, struct siginfo *info, pid_t pgid) = NULL;
int (*kh_getpgid)(pid_t pid);
int (*kh_getsid)(pid_t pid);
int (*kh_sched_setscheduler)(pid_t pid, int policy, struct sched_param *param);
long (*kh_sched_getscheduler)(pid_t pid) = NULL;
int (*kh_sched_getparam)(pid_t pid, struct sched_param *lp) = NULL;
EXPORT_SYMBOL(kh_kill_all);
EXPORT_SYMBOL(kh_kill_proc_info);
EXPORT_SYMBOL(kh_kill_pg_info);
EXPORT_SYMBOL(kh_getpgid);
EXPORT_SYMBOL(kh_getsid);
EXPORT_SYMBOL(kh_sched_setscheduler);
EXPORT_SYMBOL(kh_sched_getscheduler);
EXPORT_SYMBOL(kh_sched_getparam);

/* kerrighed_task (to be removed) */
struct task_struct;

void (*kh_copy_process)(struct task_struct *tsk);
EXPORT_SYMBOL(kh_copy_process);

/* task container */
struct task_ctnr_object;

void (*kh_task_get)(struct task_ctnr_object *obj);
void (*kh_task_put)(struct task_ctnr_object *obj);
int (*kh_task_alive)(struct task_ctnr_object *obj);
struct task_ctnr_object * (*kh_task_readlock)(pid_t pid);
struct task_ctnr_object * (*kh_task_writelock)(pid_t pid);
struct task_ctnr_object * (*kh_task_writelock_nested)(pid_t pid);
void (*kh_task_unlock)(pid_t pid);
struct task_struct* (*kh_malloc_task_struct)(pid_t pid);
void (*kh_free_task_struct)(struct task_struct *task);
EXPORT_SYMBOL(kh_task_get);
EXPORT_SYMBOL(kh_task_put);
EXPORT_SYMBOL(kh_task_alive);
EXPORT_SYMBOL(kh_task_readlock);
EXPORT_SYMBOL(kh_task_writelock);
EXPORT_SYMBOL(kh_task_writelock_nested);
EXPORT_SYMBOL(kh_task_unlock);
EXPORT_SYMBOL(kh_malloc_task_struct);
EXPORT_SYMBOL(kh_free_task_struct);

/* pid location */
int (*kh_set_pid_location)(pid_t pid, kerrighed_node_t node);
kerrighed_node_t (*kh_lock_pid_location)(pid_t pid);
void (*kh_unlock_pid_location)(pid_t pid);
EXPORT_SYMBOL(kh_set_pid_location);
EXPORT_SYMBOL(kh_lock_pid_location);
EXPORT_SYMBOL(kh_unlock_pid_location);

/* exit */
void (*kh_release_task)(struct task_struct *p);
EXPORT_SYMBOL(kh_release_task);

/* signal struct sharing */
struct signal_struct;

struct signal_struct * (*kh_malloc_signal_struct)(struct task_struct *task,
						  int need_update) = NULL;
void (*kh_share_signal)(struct task_struct *task);
pid_t (*kh_exit_signal)(struct task_struct *task);
struct signal_struct * (*kh_signal_struct_readlock)(pid_t tgid) = NULL;
struct signal_struct * (*kh_signal_struct_writelock)(pid_t tgid) = NULL;
void (*kh_signal_struct_unlock)(pid_t tgid) = NULL;
EXPORT_SYMBOL(kh_malloc_signal_struct);
EXPORT_SYMBOL(kh_share_signal);
EXPORT_SYMBOL(kh_exit_signal);
EXPORT_SYMBOL(kh_signal_struct_readlock);
EXPORT_SYMBOL(kh_signal_struct_writelock);
EXPORT_SYMBOL(kh_signal_struct_unlock);


/* vanilla Linux symbols */

#include <linux/sched.h>
#include <linux/syscalls.h>
#include <linux/pid.h>
#include <kerrighed/task.h>
#include <kerrighed/signal.h>

/* remote syscalls */
EXPORT_SYMBOL(__kill_pg_info);
EXPORT_SYMBOL(sys_getpgid);
EXPORT_SYMBOL(sys_getsid);
EXPORT_SYMBOL(sys_sched_getscheduler);
EXPORT_SYMBOL(sys_sched_getparam);

/* infrastructure */
/* task_struct allocation outside kernel/fork.c */
EXPORT_SYMBOL(__put_task_struct); /* Needed since put_task_struct is a macro */
#ifndef __HAVE_ARCH_TASK_STRUCT_ALLOCATOR
EXPORT_SYMBOL(task_struct_cachep);
#endif
EXPORT_SYMBOL(signal_cachep);

EXPORT_SYMBOL(pid_task);
